@@ -1335,7 +1335,8 @@ class SaleLine(ModelSQL, ModelView):
                         self.unit)
                     if move.invoiced_quantity < move.quantity:
                         stock_moves.append(move)
-        invoice_line.stock_moves = stock_moves
+        if self.sale.shipment_method == 'order':
+            invoice_line.stock_moves = stock_moves
 
         skip_ids = set(l.id for i in self.sale.invoices_recreated
             for l in i.lines)
@@ -1345,10 +1346,12 @@ class SaleLine(ModelSQL, ModelView):
             if old_invoice_line.id not in skip_ids:
                 quantity -= Uom.compute_qty(old_invoice_line.unit,
                     old_invoice_line.quantity, self.unit)
-        invoice_line.quantity = quantity
 
-        if invoice_line.quantity <= 0.0:
+        rounding = self.unit.rounding if self.unit else 0.01
+        invoice_line.quantity = Uom.round(quantity, rounding)
+        if invoice_line.quantity <= 0:
             return []
+
         invoice_line.unit = self.unit
         invoice_line.product = self.product
         invoice_line.unit_price = self.unit_price
@@ -1403,7 +1406,8 @@ class SaleLine(ModelSQL, ModelView):
         invoice_lines = []
         if self.sale.shipment_method == 'order':
             quantity = abs(self.quantity)
-            invoice_lines = self.invoice_lines
+            if self.sale.invoice_method == 'order':
+                invoice_lines = self.invoice_lines
         else:
             quantity = 0.0
             for invoice_line in self.invoice_lines:
@@ -1418,8 +1422,11 @@ class SaleLine(ModelSQL, ModelView):
             if move.id not in skip_ids:
                 quantity -= Uom.compute_qty(move.uom, move.quantity,
                     self.unit)
-        if quantity <= 0.0:
+
+        quantity = Uom.round(quantity, self.unit.rounding)
+        if quantity <= 0:
             return
+
         if not self.sale.party.customer_location:
             self.raise_user_error('customer_location_required', {
                     'sale': self.sale.rec_name,
@@ -1673,8 +1680,9 @@ class ReturnSale(Wizard):
         return_sales = Sale.copy(sales)
         for sale in return_sales:
             for line in sale.lines:
-                line.quantity *= -1
-                line.save()
+                if line.type == 'line':
+                    line.quantity *= -1
+                    line.save()
         data = {'res_id': [s.id for s in return_sales]}
         if len(return_sales) == 1:
             action['views'].reverse()
